import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:responsive_sizer/responsive_sizer.dart';

class TextFieldPro extends StatefulWidget {
  final String? initialValue;
  final String? hintText;
  final String? label;
  final Widget? labelWidget;
  final bool? isPassword;
  final String? prefix;
  final Widget? prefixIcon;
  final Widget? suffix;
  final Widget? suffixIcon;
  final EdgeInsets? padding;
  final List<TextInputFormatter>? inputFormatters;
  final ValueChanged<String>? onChangeTextField;
  final FormFieldSetter<String>? onSavedTextField;
  final FormFieldValidator<String>? validator;
  final TextInputType? keyboardType;
  final String? counterText;
  final int? maxLength;
  final VoidCallback? onEditingComplete;
  final GestureTapCallback? onTap;
  final bool? isValidate;
  final bool? isRequired;
  final Color? borderColor;
  final String? errorMessage;
  final IconData? errorIcon;
  final Color? errorBackgroundColor;
  final TextEditingController? controller;
  final String? errorbg;
  final Widget? errorWidget;
  final bool? readonly;
  const TextFieldPro(
      {Key? key,
      this.readonly = false,
      this.errorWidget,
      this.errorbg = "1",
      this.hintText,
      this.padding,
      this.controller,
      this.label,
      this.labelWidget,
      this.prefixIcon,
      this.suffixIcon,
      this.isPassword = false,
      this.isRequired = false,
      this.prefix,
      this.suffix,
      this.isValidate = false,
      this.inputFormatters,
      this.onEditingComplete,
      this.validator,
      this.onTap,
      this.onChangeTextField,
      this.onSavedTextField,
      this.keyboardType,
      this.counterText,
      this.maxLength,
      this.borderColor,
      this.errorMessage,
      this.errorIcon,
      this.errorBackgroundColor,
      this.initialValue})
      : super(key: key);

  @override
  State<TextFieldPro> createState() => _TextFieldProState();
}

class _TextFieldProState extends State<TextFieldPro> {
  bool isNull = true;

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        TextFormField(
          enabled: widget.readonly == true ? false : true,
          // readOnly: true,
          initialValue: widget.controller == null ? widget.initialValue : null,
          controller: widget.controller,
          style: TextStyle(fontSize: 14.sp, fontWeight: FontWeight.w500),
          scrollPadding:
              EdgeInsets.only(bottom: MediaQuery.of(context).viewInsets.bottom),
          keyboardType: widget.keyboardType,
          inputFormatters: widget.inputFormatters,
          onEditingComplete: widget.onEditingComplete,
          validator: widget.validator,
          obscureText: widget.isPassword!,
          onTap: widget.onTap,
          decoration: InputDecoration(
            filled: true,
            fillColor: widget.initialValue == "" ||
                    widget.controller!.text == "" && isNull == true
                ? const Color(0xffF2F2F2)
                : Colors.transparent,
            contentPadding: (() {
              if (widget.padding != null) {
                return widget.padding;
              } else {
                return EdgeInsets.only(
                    top: 25.sp,
                    left: 15.sp,
                    bottom: widget.prefixIcon == null ? 0 : 10.sp);
              }
            }()),
            border: OutlineInputBorder(
                borderSide: const BorderSide(color: Colors.red),
                borderRadius: BorderRadius.circular(10.sp)),
            hintText: widget.hintText,
            hintStyle:
                const TextStyle(fontSize: 14, fontWeight: FontWeight.w500),
            label: (() {
              if (widget.labelWidget != null) {
                return widget.labelWidget;
              } else {
                return Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                      widget.label != null ? '${widget.label}' : '',
                      style: TextStyle(
                        fontSize: 15.sp,
                        color: widget.isValidate == false
                            ? Colors.black
                            : Colors.red,
                      ),
                    ),
                    if (widget.isRequired == true)
                      Text(
                        ' *',
                        style: TextStyle(color: Colors.red, fontSize: 14.sp),
                      ),
                  ],
                );
              }
            }()),
            prefix: Padding(
              padding: EdgeInsets.only(right: 5.sp, left: 5.sp),
              child: Text(
                widget.prefix ?? "",
                style: Theme.of(context)
                    .textTheme
                    .headline1!
                    .copyWith(fontSize: 15.sp),
              ),
            ),
            prefixIcon: widget.prefixIcon,
            suffixIcon: widget.suffixIcon,
            suffix: widget.suffix,
            counterText: widget.counterText,
            enabledBorder: OutlineInputBorder(
              borderRadius: BorderRadius.all(Radius.circular(10.sp)),
              borderSide: BorderSide(
                  width: 1.5.sp,
                  color: widget.isValidate == false
                      ? isNull == true
                          ? const Color(0xffF2F2F2)
                          : const Color(0xff000000)
                      : Colors.red),
            ),
            focusedBorder: OutlineInputBorder(
              borderRadius: BorderRadius.all(Radius.circular(10.sp)),
              borderSide: BorderSide(
                width: 1.5.sp,
                color: (widget.isValidate == false)
                    ? widget.borderColor ?? Colors.black
                    : Colors.red,
              ),
            ),
            errorBorder: OutlineInputBorder(
              borderRadius: BorderRadius.all(Radius.circular(10.sp)),
              borderSide: BorderSide(
                width: 1.5.sp,
                color: Colors.red,
              ),
            ),
            focusedErrorBorder: OutlineInputBorder(
              borderRadius: BorderRadius.all(Radius.circular(10.sp)),
              borderSide: BorderSide(
                width: 1.5.sp,
                color: Colors.red,
              ),
            ),
          ),
          maxLength: widget.maxLength,
          onChanged: (value) {
            if (value.isEmpty) {
              setState(() {
                isNull = true;
              });
            } else {
              setState(() {
                isNull = false;
              });
            }
            if (widget.onChangeTextField != null) {
              widget.onChangeTextField!(value);
            }
          },
          onSaved: widget.onSavedTextField,
        ),
        if (widget.errorbg != "1")
          SizedBox(
            height: 10.sp,
          ),
        widget.isValidate == false
            ? Container()
            : widget.errorWidget != null
                ? widget.errorWidget!
                : Container(
                    padding: EdgeInsets.all(8.sp),
                    margin: EdgeInsets.symmetric(vertical: 5.sp),
                    decoration: widget.errorbg != "1"
                        ? BoxDecoration(
                            color: Colors.red[100],
                            borderRadius: BorderRadius.circular(5),
                          )
                        : const BoxDecoration(),
                    child: Row(
                      children: [
                        widget.errorIcon != null
                            ? Icon(
                                widget.errorIcon,
                                size: 18.sp,
                                color: (() {
                                  if (widget.errorBackgroundColor != null) {
                                    return widget.errorBackgroundColor;
                                  } else {
                                    return Colors.red;
                                  }
                                }()),
                              )
                            : Transform(
                                alignment: Alignment.center,
                                transform: Matrix4.rotationZ(3.1415),
                                child: Icon(
                                  (() {
                                    if (widget.errorIcon != null) {
                                      return widget.errorIcon;
                                    } else {
                                      return Icons.info_rounded;
                                    }
                                  }()),
                                  size: 18.sp,
                                  color: (() {
                                    if (widget.errorBackgroundColor != null) {
                                      return widget.errorBackgroundColor;
                                    } else {
                                      return Colors.red;
                                    }
                                  }()),
                                ),
                              ),
                        SizedBox(
                          width: 9.sp,
                        ),
                        Expanded(
                          child: Text(
                              widget.errorMessage != null
                                  ? widget.errorMessage!
                                  : "",
                              style: TextStyle(
                                fontSize: 14.sp,
                                color: Colors.red,
                                fontWeight: FontWeight.normal,
                              )),
                        ),
                      ],
                    ),
                  )
      ],
    );
  }
}
